@@ -21,25 +21,55 @@ along with Cecilia 5.  If not, see <http://www.gnu.org/licenses/>.
 """
 
 BaseModule_API = """
+Every module must be derived from the BaseModule class. The BaseModule's internals
+create the links between the interface and the module's processing. A Cecilia module
+must be declared like this:
+
+class Module(BaseModule):
+    def __init__(self):
+        BaseModule.__init__(self)
+        # Here comes the processing chain...
+
+The last object of the processing chain (ie the one producing the output sound) must be
+called "self.out". The audio server gets the sound from this variable and sends it to the
+Post-Processing plugins and from there, to the soundcard.
+
+Example of a typical output variable, where "self.snd" is the dry sound and "self.dsp" is 
+the processed sound. "self.drywet" is a mixing slider and "self.env" is the overall gain 
+from a grapher's line:
+
+self.out = Interp(self.snd, self.dsp, self.drywet, mul=self.env)
+
 Public attributes:
 
 self.sr : Cecilia's current sampling rate.
 self.nchnls : Cecilia's current number of channels.
 self.totalTime : Cecilia's current duration.
 self.number_of_voices : Number of voices from the cpoly widget.
-self.polyphony_spread : Spread factor from the cpoly widget.
+self.polyphony_spread : List of transposition factors from the cpoly widget.
+self.polyphony_scaling : Amplitude value according to polyphony number of voices
 
 Public methods:
 
 self.addFilein(name) : Creates a SndTable object from the name of a cfilein widget.
 self.addSampler(name, pitch, amp) : Creates a sampler/looper from the name of a csampler widget.
+self.getSamplerDur(name) : Returns the duration of the sound used by the sampler `name`. 
 self.duplicate(seq, num) : Duplicates elements in a sequence according to the `num` parameter.
-self.setGlobalSeed(x) : Sets the Server's global seed used by objects in the random family.
+self.setGlobalSeed(x) : Sets the Server's global seed used by objects from the random family.
+
+"""
+
+Interface_API = """
+The next functions describes every widgets available for building a Cecilia5 interface.
 
 """
 
 def cfilein(name="filein", label="Audio", help=""):
     """
+    Initline:
+    
+    cfilein(name="filein", label="Audio", help="")
+    
     Description:
     
     This interactive menu allows the user to import a soundfile into the 
@@ -81,6 +111,10 @@ def cfilein(name="filein", label="Audio", help=""):
 
 def csampler(name="sampler", label="Audio", help=""):
     """
+    Initline:
+
+    csampler(name="sampler", label="Audio", help="")
+    
     Description:
     
     This menu allows the user to choose a soundfile for processing in the 
@@ -136,26 +170,30 @@ def csampler(name="sampler", label="Audio", help=""):
 
 def cpoly(name="poly", label="Polyphony", min=1, max=10, init=1, help=""):
     """
+    Initline:
+
+    cpoly(name="poly", label="Polyphony", min=1, max=10, init=1, help="")
+    
     Description:
     
     cpoly is a widget conceived to help manage the voice polyphony of a 
     module. cpoly comes with a popup menu that allows the user to choose how 
     many instances (voices) of a process will be simultaneously playing. It 
-    also provides a mini slider to adjust the voice spread of those different 
-    voices.
+    also provides another popup to choose the type of polyphony (phasing, chorus,
+    out-of-tune or one of the provided chords).
 
     cpoly has two values that are passed to the processing module: the number 
     of voices and the voice spread. The number of voices can be collected 
-    using `self.number_of_voices`. To access the voice deviation factor use 
-    `self.polyphony_spread`.
+    using `self.number_of_voices`. `self.polyphony_spread` gives access to 
+    the transposition factors defined by the type of polyphony.
 
     If a csampler is used, you don't need to take care of polyphony, it's 
     automatically handled inside the csampler. Without a csampler, user can 
-    retrieve polyphony popup and slider values with these builtin reserved 
+    retrieve polyphony popups values with these builtin reserved 
     variables :
         
         self.number_of_voices : int. Number of layers of polyphony.
-        self.polyphony_spread : float. Deviation factor.
+        self.polyphony_spread : list of floats. Transposition factors.
     
     No more than one `cpoly` can be declared in a module.
 
@@ -192,6 +230,11 @@ def cpoly(name="poly", label="Polyphony", min=1, max=10, init=1, help=""):
 def cgraph(name="graph", label="Envelope", min=0.0, max=1.0, rel="lin", table=False, size=8192, 
             unit="x", curved=False, func=[(0, 0.), (.01, 1), (.99, 1), (1, 0.)], col="red"):
     """
+    Initline:
+
+    cgraph(name="graph", label="Envelope", min=0.0, max=1.0, rel="lin", table=False, size=8192, 
+                unit="x", curved=False, func=[(0, 0.), (.01, 1), (.99, 1), (1, 0.)], col="red")
+
     Description:
     
     A graph line represents the evolution of a variable during a Cecilia 
@@ -249,8 +292,13 @@ def cgraph(name="graph", label="Envelope", min=0.0, max=1.0, rel="lin", table=Fa
     return dic
     
 def cslider(name="slider", label="Pitch", min=20.0, max=20000.0, init=1000.0, rel="lin", res="float", 
-            gliss=0.025, unit="x", up=False, func=None, midictl=None, col="red", help=""):
+            gliss=0.025, unit="x", up=False, func=None, midictl=None, half=False, col="red", help=""):
     """
+    Initline:
+
+    cslider(name="slider", label="Pitch", min=20.0, max=20000.0, init=1000.0, rel="lin", res="float", 
+                gliss=0.025, unit="x", up=False, func=None, midictl=None, half=False, col="red", help="")
+
     Description:
 
     When created, the slider is stacked in the slider pane of the main Cecilia
@@ -297,6 +345,9 @@ def cslider(name="slider", label="Pitch", min=20.0, max=20000.0, init=1000.0, re
         midictl : int 
             Automatically map a midi controller to this slider. 
             Defaults to None.
+        half : boolean
+            Determines if the slider is full-width or half-width. Set to True
+            to get half-width slider. Defaults to False.
         col : str
             Color of the widget.
         help : str
@@ -316,6 +367,7 @@ def cslider(name="slider", label="Pitch", min=20.0, max=20000.0, init=1000.0, re
     dic["unit"] = unit
     dic["up"] = up
     dic["midictl"] = midictl
+    dic["half"] = half
     dic["col"] = col
     dic["help"] = help
     return dic
@@ -323,6 +375,11 @@ def cslider(name="slider", label="Pitch", min=20.0, max=20000.0, init=1000.0, re
 def crange(name="range", label="Pitch", min=20.0, max=20000.0, init=[500.0, 2000.0], rel="log", 
            res="float", gliss=0.025, unit="x", up=False, func=None, midictl=None, col="red", help=""):
     """
+    Initline:
+
+    crange(name="range", label="Pitch", min=20.0, max=20000.0, init=[500.0, 2000.0], rel="log", 
+               res="float", gliss=0.025, unit="x", up=False, func=None, midictl=None, col="red", help="")
+
     Description:
     
     When created, the range slider is stacked in the slider pane of the main 
@@ -401,13 +458,19 @@ def crange(name="range", label="Pitch", min=20.0, max=20000.0, init=[500.0, 2000
     dic["unit"] = unit
     dic["up"] = up
     dic["midictl"] = midictl
+    dic["half"] = False
     dic["col"] = col
     dic["help"] = help
     return dic
 
-def csplitter(name="splitter", label="Pitch", min=20.0, max=20000.0, init=[500.0, 2000.0, 5000.0], rel="log",
-              res="float", gliss=0.025, unit="x", up=False, num_knobs=3, col="red", help=""):
+def csplitter(name="splitter", label="Pitch", min=20.0, max=20000.0, init=[500.0, 2000.0, 5000.0], 
+              rel="log", res="float", gliss=0.025, unit="x", up=False, num_knobs=3, col="red", help=""):
     """
+    Initline:
+
+    csplitter(name="splitter", label="Pitch", min=20.0, max=20000.0, init=[500.0, 2000.0, 5000.0], 
+                  rel="log", res="float", gliss=0.025, unit="x", up=False, num_knobs=3, col="red", help="")
+
     Description:
     
     When created, the splitter is stacked in the slider pane of the main 
@@ -471,12 +534,19 @@ def csplitter(name="splitter", label="Pitch", min=20.0, max=20000.0, init=[500.0
     dic["up"] = up
     dic["num_knobs"] = num_knobs
     dic["midictl"] = None
+    dic["half"] = False
     dic["col"] = col
     dic["help"] = help
     return dic
 
-def ctoggle(name="toggle", label="Start/Stop", init=True, rate="k", col="red", help=""):
+def ctoggle(name="toggle", label="Start/Stop", init=True, rate="k", stack=False, col="red", help=""):
     """
+    Initline:
+
+    ctoggle(name="toggle", label="Start/Stop", init=True, rate="k", stack=False, col="red", help="")
+
+    Description:
+
     If `rate` argument is set to "i", a built-in reserved variable is created 
     at initialization time. The variable's name is constructed like this :
         
@@ -507,6 +577,9 @@ def ctoggle(name="toggle", label="Start/Stop", init=True, rate="k", col="red", h
             Indicates if the toggle is handled at initialization time only 
             ("i") with a reserved variable or with a function ("k") that can 
             be called at any time during playback.
+        stack : boolean
+            If True, the toggle will be added on the same row as the last toogle
+            with stack=True and a label not empty. Defaults to False
         col : str
             Color of the widget.
         help : str
@@ -518,6 +591,7 @@ def ctoggle(name="toggle", label="Start/Stop", init=True, rate="k", col="red", h
     dic["label"] = label
     dic["init"] = init
     dic["rate"] = rate
+    dic['stack'] = stack
     dic["col"] = col
     dic["help"] = help
     return dic
@@ -525,6 +599,13 @@ def ctoggle(name="toggle", label="Start/Stop", init=True, rate="k", col="red", h
 def cpopup(name="popup", label="Chooser", value=["1", "2", "3", "4"],
            init="1", rate="k", col="red", help=""):
     """
+    Initline:
+
+    cpopup(name="popup", label="Chooser", value=["1", "2", "3", "4"],
+               init="1", rate="k", col="red", help="")
+
+    Description:
+
     If `rate` argument is set to "i", two built-in reserved variables are 
     created at initialization time. The variables' names are constructed 
     like this :
@@ -578,6 +659,12 @@ def cpopup(name="popup", label="Chooser", value=["1", "2", "3", "4"],
 
 def cbutton(name="button", label="Trigger", col="red", help=""):
     """
+    Initline:
+
+    cbutton(name="button", label="Trigger", col="red", help="")
+
+    Description:
+
     When the button is clicked, a function is called with the current state 
     of the button as argument.
     
@@ -606,8 +693,16 @@ def cbutton(name="button", label="Trigger", col="red", help=""):
     dic["help"] = help
     return dic
 
-def cgen(name="gen", label="Wave shape", init=[1,0,.3,0,.2,0,.143,0,.111], rate="k", popup=None, col="red", help=""):
+def cgen(name="gen", label="Wave shape", init=[1,0,.3,0,.2,0,.143,0,.111], 
+         rate="k", popup=None, col="red", help=""):
     """
+    Initline:
+
+    cgen(name="gen", label="Wave shape", init=[1,0,.3,0,.2,0,.143,0,.111], 
+             rate="k", popup=None, col="red", help="")
+
+    Description:
+
     Widget used to create a list of floating-point values.
     
     If `rate` argument is set to "i", a built-in reserved variable is created 
