@@ -26,6 +26,7 @@ import Variables as vars
 from API_interface import *
 import unicodedata
 from subprocess import Popen
+from pyolib._wxwidgets import SpectrumDisplay
 
 def setVar(var, value):
     vars.CeciliaVar[var] = value
@@ -60,10 +61,19 @@ def writeVarToDisk():
 def startCeciliaSound(timer=True, rec=False):
     # Check if soundfile is loaded
     for key in getVar("userInputs").keys():
-        if not os.path.isfile(getVar("userInputs")[key]['path']):
-            showErrorDialog('"%s", no input sound file!' % getControlPanel().getCfileinFromName(key).label, 'Please load one...')
-            getControlPanel().getCfileinFromName(key).onLoadFile()
+        if getVar("userInputs")[key]['mode'] == 0:
+            if not os.path.isfile(getVar("userInputs")[key]['path']):
+                showErrorDialog('"%s", no input sound file!' % getControlPanel().getCfileinFromName(key).label, 'Please load one...')
+                getControlPanel().getCfileinFromName(key).onLoadFile()
     getControlPanel().resetMeter()
+    if getVar('spectrumFrame') != None:
+        try:
+            getVar('spectrumFrame')._destroy(None)
+        except:
+            getVar('interface').menubar.spectrumSwitch(False)
+            setVar('showSpectrum', 0)
+        finally:
+            setVar('spectrumFrame', None)
     getVar("audioServer").shutdown()
     getVar("audioServer").reinit()
     getVar("audioServer").boot()
@@ -74,6 +84,11 @@ def startCeciliaSound(timer=True, rec=False):
     getVar("grapher").toolbar.convertSlider.Hide()
     getVar("presetPanel").presetChoice.setEnable(False)
     getVar("audioServer").start(timer=timer, rec=rec)
+    if getVar('showSpectrum'):
+        f = SpectrumDisplay(None, getVar("audioServer").spectrum)
+        getVar("audioServer").spectrum._setViewFrame(f)
+        setVar('spectrumFrame', f)
+        f.Show()
     getVar("grapher").toolbar.loadingMsg.SetForegroundColour(TITLE_BACK_COLOUR)
     wx.CallAfter(getVar("grapher").toolbar.loadingMsg.Refresh)
 
@@ -368,6 +383,7 @@ def loadPresetFromDict(preset):
         getVar("presetPanel").setLabel(preset)
         getVar("grapher").getPlotter().draw()
         setVar("currentModule", currentModule)
+        getVar("grapher").setTotalTime(getVar("totalTime"))
 
 def savePresetToDict(presetName):
     presetDict = dict()
@@ -429,6 +445,7 @@ def savePresetToDict(presetName):
 
 def completeUserInputsDict():
     for i in getVar("userInputs"):
+        getVar("userInputs")[i]['mode'] = 0
         if getVar("userInputs")[i]['type'] == 'csampler':
             cfilein = getControlPanel().getCfileinFromName(i)
             getVar("userInputs")[i]['off'+cfilein.getName()] = cfilein.getOffset()
