@@ -86,19 +86,31 @@ class CeciliaMainFrame(wx.Frame):
         num_snds = len(cfileins[0].fileMenu.choice)
         dlg = wx.ProgressDialog("Batch processing on sound folder", "", maximum = num_snds, parent=self,
                                style = wx.PD_APP_MODAL | wx.PD_AUTO_HIDE | wx.PD_SMOOTH)
-        if CeciliaLib.getVar("systemPlatform") == "win32":
-            dlg.SetSize((600, 125))
-        else:
-            dlg.SetSize((600,100))
+        dlg.SetMinSize((600,-1))
+        dlg.SetClientSize((600,100))
         count = 0
+        totaltime = CeciliaLib.getVar("totalTime")
         for snd in cfileins[0].fileMenu.choice:
             cfileins[0].onSelectSound(-1, snd)
+            if CeciliaLib.getVar("useSoundDur"):
+                cfileins[0].setTotalTime()
             path, dump = os.path.split(cfileins[0].filePath)
             name, ext = os.path.splitext(snd)
-            if ext in [".wav", ".wave", ".WAV", ".WAVE", ".Wav", ".Wave"]:
+            lext = ext.lower()
+            if lext in [".wav", ".wave"]:
                 CeciliaLib.setVar('audioFileType', "wav")
-            else:
+            elif lext in [".aif", ".aiff", ".aifc"]:
                 CeciliaLib.setVar('audioFileType', "aif")
+            elif lext in [".ogg"]:
+                CeciliaLib.setVar('audioFileType', "ogg")
+            elif lext in [".flac"]:
+                CeciliaLib.setVar('audioFileType', "flac")
+            elif lext in [".au"]:
+                CeciliaLib.setVar('audioFileType', "au")
+            elif lext in [".sd2"]:
+                CeciliaLib.setVar('audioFileType', "sd2")
+            elif lext in [".caf"]:
+                CeciliaLib.setVar('audioFileType', "caf")
             if not os.path.isdir(os.path.join(path, folderName)):
                 os.mkdir(os.path.join(path, folderName))
             filename = os.path.join(path, folderName, "%s-%s%s" % (name, folderName, ext))
@@ -107,6 +119,10 @@ class CeciliaMainFrame(wx.Frame):
             CeciliaLib.getControlPanel().onBatchProcessing(filename)
             while (CeciliaLib.getVar("audioServer").isAudioServerRunning()):
                 time.sleep(.1)
+        if CeciliaLib.getVar("useSoundDur"):
+            CeciliaLib.getControlPanel().setTotalTime(totaltime)
+            CeciliaLib.getControlPanel().updateDurationSlider()
+
         dlg.Destroy()
         CeciliaLib.setVar('audioFileType', old_file_type)
 
@@ -123,10 +139,8 @@ class CeciliaMainFrame(wx.Frame):
         num_presets = len(presets)
         dlg = wx.ProgressDialog("Batch processing on preset sequence", "", maximum = num_presets, parent=self,
                                style = wx.PD_APP_MODAL | wx.PD_AUTO_HIDE | wx.PD_SMOOTH)
-        if CeciliaLib.getVar("systemPlatform") == "win32":
-            dlg.SetSize((600, 125))
-        else:
-            dlg.SetSize((600,100))
+        dlg.SetMinSize((600,-1))
+        dlg.SetClientSize((600,100))
         count = 0
         for preset in presets:
             CeciliaLib.loadPresetFromDict(preset)
@@ -156,15 +170,11 @@ class CeciliaMainFrame(wx.Frame):
         f.CenterOnScreen()
         f.Show()
 
-    def onSelectOutputFilename(self):
-        if CeciliaLib.getVar("audioFileType") == 'wav':
-            wildcard = "Wave file|*.wav;*.wave;*.WAV;*.WAVE;*.Wav;*.Wave|" \
-                       "All files|*.*"
-        else:
-            wildcard = "AIFF file|*.aif;*.aiff;*.aifc;*.AIF;*.AIFF;*.Aif;*.Aiff|" \
-                       "All files|*.*"
-        
-        file = CeciliaLib.saveFileDialog(self, wildcard, type='Save audio')
+    def onUseSoundDuration(self, evt):
+        CeciliaLib.setVar("useSoundDur", evt.GetInt())
+
+    def onSelectOutputFilename(self):        
+        file = CeciliaLib.saveFileDialog(self, AUDIO_FILE_WILDCARD, type='Save audio')
         if file != None:
             CeciliaLib.setVar("saveAudioFilePath", os.path.split(file)[0])
         return file
@@ -213,8 +223,11 @@ class CeciliaMainFrame(wx.Frame):
             for item in self.menubar.openRecentMenu.GetMenuItems():
                 self.menubar.openRecentMenu.DeleteItem(item)
             for file in recentFiles:
-                self.menubar.openRecentMenu.Append(subId2, file)
-                subId2 += 1
+                try:
+                    self.menubar.openRecentMenu.Append(subId2, file)
+                    subId2 += 1
+                except:
+                    pass
 
     def onOpen(self, event, builtin=False):
         if isinstance(event, wx.CommandEvent):
@@ -324,6 +337,12 @@ class CeciliaMainFrame(wx.Frame):
                 cfilein.onLoadFile(snds[i])
         wx.CallAfter(ceciliaInterface.OnSize, wx.PaintEvent(wx.ID_ANY))
 
+    def onShowSpectrum(self, event):
+        if event.GetInt():
+            CeciliaLib.setVar('showSpectrum', 1)
+        else:
+            CeciliaLib.setVar('showSpectrum', 0)
+            
     def onQuit(self, event):
         if not CeciliaLib.closeCeciliaFile(self):
             return
@@ -333,7 +352,14 @@ class CeciliaMainFrame(wx.Frame):
             pass
         if CeciliaLib.getVar("audioServer").isAudioServerRunning():
             CeciliaLib.getVar("audioServer").stop()
-            time.sleep(.1)
+            time.sleep(.2)
+        if CeciliaLib.getVar('spectrumFrame') != None:
+            try:
+                CeciliaLib.getVar('spectrumFrame')._destroy(None)
+            except:
+                pass
+            finally:
+                CeciliaLib.setVar('spectrumFrame', None)
         self.doc_frame.Destroy()
         self.closeInterface()
         CeciliaLib.writeVarToDisk()
