@@ -19,9 +19,11 @@ along with Cecilia 5.  If not, see <http://www.gnu.org/licenses/>.
 """
 
 import wx, time, random, bisect
+from wx.lib.stattext import GenStaticText
 import CeciliaPlot as plot
 import math, copy
 from constants import *
+from Grapher_parser import *
 import CeciliaLib
 from Widgets import *
 from types import ListType, TupleType
@@ -152,6 +154,13 @@ class Line:
         self.lines = []
         self.dataToDraw = []
         self.initData = self.getLineState()
+        self.modified = True
+
+    def getModified(self):
+        return self.modified
+        
+    def setModified(self, state):
+        self.modified = state
 
     def getLineState(self):
         dict = {'data': self.normalize(),
@@ -164,6 +173,7 @@ class Line:
         self.data = self.denormalize(data)
         self.curved = dict.get('curved', False)
         self.checkIfCurved()
+        self.modified = True
 
     def changeYrange(self, newrange):
         d = self.getLineState()
@@ -171,6 +181,7 @@ class Line:
         self.scale = self.yrange[1] - self.yrange[0]
         self.offset = self.yrange[0]
         self.setLineState(d)
+        self.modified = True
 
     def getColour(self):
         return self.colour
@@ -187,35 +198,43 @@ class Line:
     def setData(self, list):
         self.data = list
         self.checkIfCurved()
+        self.modified = True
 
     def reset(self):
         self.setLineState(self.initData)
         self.initData = self.getLineState()
+        self.modified = True
 
     def setPoint(self, point, value):
         self.data[point] = value
         self.checkIfCurved()
+        self.modified = True
 
     def move(self, list, offset):
         self.data = [[l[0] - offset[0], l[1] - offset[1]] for l in list]
         self.checkIfCurved()
+        self.modified = True
 
     def moveLog(self, list, offset):
         self.data = [[l[0] - offset[0], l[1] * offset[1]] for l in list]
         self.checkIfCurved()
+        self.modified = True
 
     def insert(self, pos, value):
         self.data.insert(pos, value)
         self.checkIfCurved()
+        self.modified = True
 
     def deletePoint(self, pos):
         del self.data[pos]
         self.checkIfCurved()
+        self.modified = True
 
     def deletePointFromPoint(self, point):
         if point in self.data:
             self.data.remove(point)
             self.checkIfCurved()
+            self.modified = True
 
     def setShow(self, state):
         self.show = state
@@ -235,6 +254,9 @@ class Line:
     def getLabel(self):
         return self.label
 
+    def setName(self, name):
+        self.name = name
+
     def getName(self):
         return self.name
 
@@ -296,6 +318,7 @@ class Line:
             self.curved = True
             self.lines = linToCosCurve(data=[p for p in self.getData()], yrange=self.getYrange(),
                                        totaldur=CeciliaLib.getVar("totalTime"), points=1024, log=self.getLog())
+        self.modified = True
 
     def checkIfCurved(self):
         if self.getCurved():
@@ -312,7 +335,6 @@ class Grapher(plot.PlotCanvas):
         self.menubarRedo = self.parent.parent.menubar.FindItemById(ID_REDO)
         self._history = []
         self._historyPoint = 0
-        self._clipboard = None
         self._tool = 0
         self._zoomed = False
         self.SetUseScientificNotation(False)
@@ -330,6 +352,7 @@ class Grapher(plot.PlotCanvas):
         self.selectedPoints = []
         self.markSelectionStart = None
         self.lineOver = None # mouse hover curve
+        self.lineOverGate = True
         self.selected = 0 # selected curve
         self._oldSelected = -1
         self._graphCreation = True
@@ -349,7 +372,7 @@ class Grapher(plot.PlotCanvas):
         win = wx.FindWindowAtPointer()
         if win != None:
             win = win.GetTopLevelParent()
-            if win not in [CeciliaLib.getVar("mainFrame"), CeciliaLib.getVar("interface")]:
+            if win not in [CeciliaLib.getVar("mainFrame"), CeciliaLib.getVar("interface"), CeciliaLib.getVar("spectrumFrame")]:
                 win.Raise()
         event.Skip()
         
@@ -422,12 +445,30 @@ class Grapher(plot.PlotCanvas):
 
     def onCopy(self):
         line = self.getLine(self.getSelected())
-        self._clipboard = line.getLineState()
+        data = wx.TextDataObject(str(line.getLineState()))
+        if wx.TheClipboard.Open():
+            wx.TheClipboard.Clear()
+            wx.TheClipboard.SetData(data)
+            wx.TheClipboard.Close()
 
     def onPaste(self):
-        if self._clipboard:
+        if not wx.TheClipboard.IsOpened():
+            do = wx.TextDataObject()
+            wx.TheClipboard.Open()
+            success = wx.TheClipboard.GetData(do)
+            wx.TheClipboard.Close()
+            if success:
+                text = do.GetText()
+            else:
+                return
+            state = import_after_effects_automation(text)
+            if state == None:
+                try:
+                    state = eval(text)
+                except:
+                    return
             line = self.getLine(self.getSelected())
-            line.setLineState(self._clipboard)
+            line.setLineState(state)
             self.draw()
             self.onCopy()
             self.checkForHistory()
@@ -477,7 +518,8 @@ class Grapher(plot.PlotCanvas):
         else:
             self.menubarUndo.Enable(False)
             self.menubarRedo.Enable(False)
-         
+
+    # zoom() is no more used... #
     def zoom(self):
         if self._zoomed:
             minX, minY= _Numeric.minimum( self._zoomCorner1, self._zoomCorner2)
@@ -543,10 +585,8 @@ class Grapher(plot.PlotCanvas):
             self._zoomCorner1 = _Numeric.array(self.rescaleLogLin([_zoomCorner1], currentLine.getYrange(), newLine.getYrange()))[0]
             self._zoomCorner2 = _Numeric.array(self.rescaleLogLin([_zoomCorner2], currentLine.getYrange(), newLine.getYrange()))[0]
         elif not currentLine.getLog() and not newLine.getLog():
-            self._zoomCorner1 = _Numeric.array(self.rescaleLinLin([self._zoomCorner1], currentLine.getScale(), newLine.getScale(),
-                                                                                       currentLine.getOffset(), newLine.getOffset()))[0]
-            self._zoomCorner2 = _Numeric.array(self.rescaleLinLin([self._zoomCorner2], currentLine.getScale(), newLine.getScale(),
-                                                                                       currentLine.getOffset(), newLine.getOffset()))[0]
+            self._zoomCorner1 = _Numeric.array(self.rescaleLinLin([self._zoomCorner1], currentLine.getYrange(), newLine.getYrange()))[0]
+            self._zoomCorner2 = _Numeric.array(self.rescaleLinLin([self._zoomCorner2], currentLine.getYrange(), newLine.getYrange()))[0]
         elif not currentLine.getLog() and newLine.getLog():
             self._zoomCorner1 = _Numeric.array(self.rescaleLinLog([self._zoomCorner1], currentLine.getYrange(), newLine.getYrange()))[0]
             self._zoomCorner2 = _Numeric.array(self.rescaleLinLog([self._zoomCorner2], currentLine.getYrange(), newLine.getYrange()))[0]
@@ -609,7 +649,7 @@ class Grapher(plot.PlotCanvas):
                         marker = plot.PolyMarker([l.getData()[0], l.getData()[-1]], size=1.1, marker="bmp", fillcolour='black')
                     else:
                         marker = plot.PolyMarker(l.getData(), size=1.1, marker="bmp", fillcolour='black')
-                    if CeciliaLib.getVar("currentModule") != None:
+                    if CeciliaLib.getVar("currentModule") != None and l.getModified():
                         if widget_type == "graph":
                             CeciliaLib.getVar("currentModule")._graphs[l.name].setValue(data)
                         elif widget_type == "range":
@@ -618,6 +658,9 @@ class Grapher(plot.PlotCanvas):
                             CeciliaLib.getVar("currentModule")._samplers[sampler_name].setGraph(l.name, data)
                         elif widget_type == "slider":
                             CeciliaLib.getVar("currentModule")._sliders[l.name].setGraph(data)
+                        elif widget_type == "plugin_knob":
+                            CeciliaLib.getVar("audioServer").setPluginGraph(slider.getParentVPos(), slider.getKnobPos(), data)
+                        l.setModified(False)
                 else:
                     if needRedrawNonSelCurves:
                         if currentLog:
@@ -654,7 +697,6 @@ class Grapher(plot.PlotCanvas):
         gc = plot.PlotGraphics(lines, 'Title', '', '')
         self.Draw(gc, xAxis = (0, self.totaltime), yAxis = currentYrange)
         self._currentData = self.getRawData()
-        self.zoom()
 
     def OnLeave(self, event):
         self.curve = None
@@ -663,30 +705,31 @@ class Grapher(plot.PlotCanvas):
         self.draw()
 
     def OnMouseDoubleClick(self, event):
-        if self.lineOver != None and self.lineOver == self.selected:
-            line = self.data[self.lineOver]
-            line.setCurvedLine()
-            if line.getSlider() != None:
-                line.getSlider().setPlay(1)
-        else:
-            pos = list(self.GetXY(event))
-            line = self.data[self.selected]
-            points = [p[0] for p in line.getData()]
-            for i in range(len(points)-1):
-                if pos[0] > points[i] and pos[0] < points[i+1]:
-                    line.insert(i+1, pos)
-                    self.point = i+1
-            if pos[0] > points[-1]:
-                    line.insert(len(points), pos)
-                    self.point = len(points)
-            elif pos[0] < points[0]:
-                    line.insert(0, pos)
-                    self.point = 0
-            if line.getSlider() != None:
-                line.getSlider().setPlay(1)
-            self.curve = self.selected
-            self.setValuesToDraw(self._getXY(event), pos[0], pos[1])
-        self.draw()
+        if self._tool < 2:
+            if self.lineOver != None and self.lineOver == self.selected:
+                line = self.data[self.lineOver]
+                line.setCurvedLine()
+                if line.getSlider() != None:
+                    line.getSlider().setPlay(1)
+            else:
+                pos = list(self.GetXY(event))
+                line = self.data[self.selected]
+                points = [p[0] for p in line.getData()]
+                for i in range(len(points)-1):
+                    if pos[0] > points[i] and pos[0] < points[i+1]:
+                        line.insert(i+1, pos)
+                        self.point = i+1
+                if pos[0] > points[-1]:
+                        line.insert(len(points), pos)
+                        self.point = len(points)
+                elif pos[0] < points[0]:
+                        line.insert(0, pos)
+                        self.point = 0
+                if line.getSlider() != None:
+                    line.getSlider().setPlay(1)
+                self.curve = self.selected
+                self.setValuesToDraw(self._getXY(event), pos[0], pos[1])
+            self.draw()
 
     def OnMouseRightDown(self, event):
         pos = self._getXY(event)
@@ -939,9 +982,19 @@ class Grapher(plot.PlotCanvas):
                         else: maxboundary = self.data[self.curve].getData()[p+1][0]
 
                         if self.data[self.selected].getLog():
-                            offset = (self.startpos[0] - pos[0], pos[1] / self.startpos[1])
+                            if event.AltDown() and event.ShiftDown():
+                                offset = (self.startpos[0] - pos[0], 1)
+                            elif event.AltDown():
+                                offset = (0, pos[1] / self.startpos[1])
+                            else:
+                                offset = (self.startpos[0] - pos[0], pos[1] / self.startpos[1])
                         else:
-                            offset = (self.startpos[0] - pos[0], self.startpos[1] - pos[1])
+                            if event.AltDown() and event.ShiftDown():
+                                offset = (self.startpos[0] - pos[0], 0)
+                            elif event.AltDown():
+                                offset = (0, self.startpos[1] - pos[1])
+                            else:
+                                offset = (self.startpos[0] - pos[0], self.startpos[1] - pos[1])
 
                         newxpos =  self.templist[self.selectedPoints.index(p)][0] - offset[0]
                         if newxpos < minboundary: X = minboundary
@@ -997,6 +1050,7 @@ class Grapher(plot.PlotCanvas):
                             l = self.data.index(self.visibleLines[ldata[0]])
                             if self.data.index(curve) == l:
                                 self.lineOver = self.data.index(curve)
+                                self.lineOverGate = True
                 else:
                     # Check mouse over if not curved
                     currentYrange = curve.getYrange()
@@ -1009,16 +1063,24 @@ class Grapher(plot.PlotCanvas):
                         i = len(curveData) - 3
                     if distanceToSegment(checkPos, curveData[i], curveData[i+1], 0, self.totaltime, currentYrange[0], currentYrange[1], False, curve.getLog()) <= pourcent:
                         self.lineOver = self.data.index(curve)
+                        self.lineOverGate = True
                     elif distanceToSegment(checkPos, curveData[i-1], curveData[i], 0, self.totaltime, currentYrange[0], currentYrange[1], False, curve.getLog()) <= pourcent:
                         self.lineOver = self.data.index(curve)
+                        self.lineOverGate = True
                     elif distanceToSegment(checkPos, curveData[i+1], curveData[i+2], 0, self.totaltime, currentYrange[0], currentYrange[1], False, curve.getLog()) <= pourcent:
                         self.lineOver = self.data.index(curve)
+                        self.lineOverGate = True
                     else:
                         self.lineOver = None
-                self.draw()
+                if self.lineOverGate:
+                    self.draw()
+                if self.lineOver == None:
+                    self.lineOverGate = False
 
         elif self._tool == 1 and event.LeftIsDown():
             pos = self.GetXY(event)
+            if self._pencilOldPos == None:
+                self._pencilOldPos = pos
             line = self.data[self.selected]
             if pos[0] < 0.0: pos = [0.0, pos[1]]
             elif pos[0] > CeciliaLib.getVar("totalTime"): pos = [CeciliaLib.getVar("totalTime"), pos[1]]
@@ -1157,7 +1219,7 @@ class ToolBar(wx.Panel):
         self.parent = parent
         ffakePanel = wx.Panel(self, -1, size=(5, self.GetSize()[1]))
         ffakePanel.SetBackgroundColour(TITLE_BACK_COLOUR)
-        self.menu = CustomMenu(self, choice=[], size=(130,20), init=None, outFunction=self.parent.onPopupMenu)
+        self.menu = CustomMenu(self, choice=[], size=(150,20), init=None, outFunction=self.parent.onPopupMenu)
         self.menu.setBackgroundColour(TITLE_BACK_COLOUR)
         self.menu.SetToolTip(CECTooltip(TT_GRAPH_POPUP))
         self.toolbox = ToolBox(self, tools=tools, outFunction=toolFunctions)
@@ -1168,12 +1230,16 @@ class ToolBar(wx.Panel):
 
         fakePanel = wx.Panel(self, -1, size=(10, self.GetSize()[1]))
         fakePanel.SetBackgroundColour(TITLE_BACK_COLOUR)
-        self.loadingMsg = wx.StaticText(fakePanel, label="loading buffers...    ", pos=(-1, 5))
+        if CeciliaLib.getVar("systemPlatform") == "win32":
+            self.loadingMsg = GenStaticText(fakePanel, -1, label="loading buffers...    ", pos=(-1, 5))
+        else:
+            self.loadingMsg = wx.StaticText(fakePanel, -1, label="loading buffers...    ", pos=(-1, 5))            
         self.loadingMsg.SetForegroundColour(TITLE_BACK_COLOUR)
         font = self.loadingMsg.GetFont()
         font.SetFaceName(FONT_FACE)
         font.SetPointSize(MENU_FONT)
         self.loadingMsg.SetFont(font)
+        self.loadingMsg.Refresh()
 
         if CeciliaLib.getVar("moduleDescription") != '':
             helpButton = CloseBox(fakePanel, size=(18,18), pos=(25,2), outFunction=self.onShowModuleDescription, 
@@ -1209,7 +1275,7 @@ class ToolBar(wx.Panel):
         win = wx.FindWindowAtPointer()
         if win != None:
             win = win.GetTopLevelParent()
-            if win not in [CeciliaLib.getVar("mainFrame"), CeciliaLib.getVar("interface")]:
+            if win not in [CeciliaLib.getVar("mainFrame"), CeciliaLib.getVar("interface"), CeciliaLib.getVar("spectrumFrame")]:
                 win.Raise()
         event.Skip()
 
@@ -1253,6 +1319,7 @@ class CursorPanel(wx.Panel):
         self.time = 0
         self.Bind(wx.EVT_PAINT, self.OnPaint)
         self.Bind(wx.EVT_LEAVE_WINDOW, self.OnLooseFocus)
+        self.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
         self.bitmap = self.createBitmap()
 
     def createBitmap(self):
@@ -1276,10 +1343,21 @@ class CursorPanel(wx.Panel):
         win = wx.FindWindowAtPointer()
         if win != None:
             win = win.GetTopLevelParent()
-            if win not in [CeciliaLib.getVar("mainFrame"), CeciliaLib.getVar("interface")]:
+            if win not in [CeciliaLib.getVar("mainFrame"), CeciliaLib.getVar("interface"), CeciliaLib.getVar("spectrumFrame")]:
                 win.Raise()
         event.Skip()
 
+    def OnLeftDown(self, evt):
+        if not CeciliaLib.getVar("audioServer").isAudioServerRunning():
+            gap = int(self.parent.plotter.PositionUserToScreen((0,0))[0])
+            w, h = self.GetSize()
+            pos = evt.GetPosition()[0]
+            t = float(pos - gap - 4) / (w - gap * 2) * CeciliaLib.getVar("totalTime")
+            if t < 0.0:
+                t = 0.0
+            CeciliaLib.setVar('startOffset', t)
+            self.setTime(t)
+
     def OnPaint(self, evt):
         gap = int(self.parent.plotter.PositionUserToScreen((0,0))[0])
         totalTime = CeciliaLib.getVar("totalTime")
@@ -1354,6 +1432,25 @@ class CECGrapher(wx.Panel):
     def setTotalTime(self, time):
         self.plotter.setTotalTime(time)
 
+    def setSamplerLineStates(self, name, state):
+        names = ['%s %s' % (name, n) for n in ['Loop In', 'Loop Time', 'Loop X', 'Gain', 'Transpo']]
+        labels = self.toolbar.getPopupChoice()
+        if state:
+            for n in names:
+                if n not in labels:
+                    labels.append(n)
+        else:
+            for n in names:
+                if n in labels:
+                    labels.remove(n)
+        self.toolbar.setPopupChoice(labels)
+        names = ['%s%s' % (name, n) for n in ['start', 'end', 'xfade', 'gain', 'trans']]
+        if not state:
+            for line in self.plotter.getData():
+                if line.getName() in names:
+                    line.setShow(False)
+        self.plotter.draw()
+        
     def createLines(self, list):
         for l in list:
             self.createLine(l[0], l[1], l[2], l[3], l[4], l[5], l[6], l[7])
@@ -1381,13 +1478,23 @@ class CECGrapher(wx.Panel):
 
     def OnLoad(self):
         line = self.plotter.getLine(self.plotter.getSelected())
-        dlg = wx.FileDialog(self, message="Choose a file", defaultDir=os.getcwd(), 
+        dlg = wx.FileDialog(self, message="Choose a grapher file", defaultDir=CeciliaLib.getVar("grapherLinePath"), 
             defaultFile="", style=wx.OPEN | wx.CHANGE_DIR)
         if dlg.ShowModal() == wx.ID_OK:
             path = dlg.GetPath()
-            f = open(path, 'r')
-            state = eval(f.read())
-            f.close()
+            CeciliaLib.setVar("grapherLinePath", os.path.split(path)[0])
+            try:
+                f = open(path, 'r')
+                text = f.read()
+                f.close()
+            except:
+                return
+            state = import_after_effects_automation(text)
+            if state == None:
+                try:
+                    state = eval(text)
+                except:
+                    return
             line.setLineState(state)
             self.plotter.draw()
             self.plotter.checkForHistory()
