@@ -56,8 +56,10 @@ class PlayRecButtons(wx.Panel):
         self.Bind(wx.EVT_PAINT, self.OnPaint)
         self.Bind(wx.EVT_MOTION, self.OnMotion)
         self.Bind(wx.EVT_LEAVE_WINDOW, self.OnLeave)
+        self.Bind(wx.EVT_ENTER_WINDOW, self.OnEnter)
         self.Bind(wx.EVT_LEFT_DOWN, self.MouseDown)
         self.Bind(wx.EVT_LEFT_UP, self.MouseUp)
+        self.enterWithButtonDown = False
         self.playColour = SLIDER_PLAY_COLOUR_HOT
         self.recColour = SLIDER_REC_COLOUR_HOT
         self.playOver = False
@@ -94,16 +96,31 @@ class PlayRecButtons(wx.Panel):
                 self.setOverWait(1)
             self.playOver = False
             self.recOver = False
-            self.Refresh()
-            self.CaptureMouse()
+            wx.CallAfter(self.Refresh)
+            evt.Skip()
+
+    def OnEnter(self, evt):
+        if evt.ButtonIsDown(wx.MOUSE_BTN_LEFT) and not CeciliaLib.getVar("audioServer").isAudioServerRunning():
+            self.enterWithButtonDown = True
+            pos = evt.GetPosition()
+            if wx.Rect(0, 0, 20, 20).Contains(pos):
+                self.play = (self.play + 1) % 3
+                self.setPlay(self.play)
+            elif wx.Rect(20, 0, 40, 20).Contains(pos):
+                if self.rec: 
+                    self.setRec(False)
+                else: 
+                    self.setRec(True)
+            self.playOver = False
+            self.recOver = False
+            wx.CallAfter(self.Refresh)
             evt.Skip()
 
     def MouseUp(self, evt):
-        if self.HasCapture():
-            self.ReleaseMouse()
+        self.enterWithButtonDown = False
 
     def OnMotion(self, evt):
-        if not CeciliaLib.getVar("audioServer").isAudioServerRunning():
+        if not CeciliaLib.getVar("audioServer").isAudioServerRunning() and not self.enterWithButtonDown:
             pos = evt.GetPosition()
             if wx.Rect(2, 2, 17, 17).Contains(pos) and self.playOverWait:
                 self.playOver = True
@@ -112,15 +129,16 @@ class PlayRecButtons(wx.Panel):
                 self.playOver = False
                 self.recOver = True
             self.checkForOverReady(pos)
-            self.Refresh()
+            wx.CallAfter(self.Refresh)
             evt.Skip()
 
     def OnLeave(self, evt):
+        self.enterWithButtonDown = False
         self.playOver = False
         self.recOver = False
         self.playOverWait = True
         self.recOverWait = True
-        self.Refresh()
+        wx.CallAfter(self.Refresh)
         evt.Skip()
 
     def OnPaint(self, evt):
@@ -164,7 +182,7 @@ class PlayRecButtons(wx.Panel):
             if self.rec:
                 self.setRec(0)
             self.playColour = SLIDER_PLAY_COLOUR_NO_BIND
-        self.Refresh()
+        wx.CallAfter(self.Refresh)
 
     def getPlay(self):
         return self.play
@@ -256,14 +274,14 @@ class Slider(wx.Panel):
         self.pos = clamp(evt.GetPosition()[0], self.knobHalfSize, size[0]-self.knobHalfSize)
         self.value = self.scale()
         self.CaptureMouse()
-        self.Refresh()
+        wx.CallAfter(self.Refresh)
 
     def MouseMotion(self, evt):
         size = self.GetSize()
         if evt.Dragging() and evt.LeftIsDown() and self.HasCapture():
             self.pos = clamp(evt.GetPosition()[0], self.knobHalfSize, size[0]-self.knobHalfSize)
             self.value = self.scale()
-            self.Refresh()
+            wx.CallAfter(self.Refresh)
                    
     def MouseUp(self, evt):
         if self.HasCapture():
@@ -277,11 +295,11 @@ class Slider(wx.Panel):
         self.createBackgroundBitmap()
         self.createKnobMaskBitmap()
         self.clampPos()    
-        self.Refresh()
+        wx.CallAfter(self.Refresh)
 
     def clampPos(self):
         size = self.GetSize()
-        self.pos = tFromValue(self.value, self.minvalue, self.maxvalue) * (size[0] - self.knobHalfSize) + self.knobHalfSize
+        self.pos = tFromValue(self.value, self.minvalue, self.maxvalue) * (size[0] - self.knobSize) + self.knobHalfSize
         self.pos = clamp(self.pos, self.knobHalfSize, size[0]-self.knobHalfSize)
 
 class HSlider(Slider):
@@ -289,8 +307,16 @@ class HSlider(Slider):
                  valtype='float', log=False, function=None, cecslider=None):
         Slider.__init__(self, parent, minvalue, maxvalue, init, pos, size, valtype, log, function, cecslider)
         self.SetMinSize((50, 15))
-        self.knobSize = 26
-        self.knobHalfSize = 13
+        if self.cecslider:
+            if self.cecslider.half:
+                self.knobSize = 22
+                self.knobHalfSize = 11
+            else:
+                self.knobSize = 28
+                self.knobHalfSize = 14
+        else:
+            self.knobSize = 28
+            self.knobHalfSize = 14
         self.sliderHeight = 14
         self.createSliderBitmap()
         self.createKnobMaskBitmap()
@@ -302,7 +328,8 @@ class HSlider(Slider):
         self.clampPos()
         self.midictl = ''
         self.midiLearn = False
-        self.font = wx.Font(LABEL_FONT, wx.NORMAL, wx.ITALIC, wx.LIGHT, face=FONT_FACE)
+        self.openSndCtrl = ''
+        self.font = wx.Font(LABEL_FONT-2, wx.NORMAL, wx.ITALIC, wx.LIGHT, face=FONT_FACE)
 
         self.mario = 0
         self.useMario = False
@@ -315,7 +342,7 @@ class HSlider(Slider):
         self.createKnobMaskBitmap()
         self.createBackgroundBitmap()
         self.createKnobBitmap()
-        self.Refresh()
+        wx.CallAfter(self.Refresh)
 
     def createBackgroundBitmap(self):
         w,h = self.GetSize()
@@ -352,11 +379,15 @@ class HSlider(Slider):
     def setMidiCtl(self, str):
         self.midictl = str
         self.midiLearn = False
-        
+
     def inMidiLearnMode(self):
         self.midiLearn = True
-        self.Refresh()
-        
+        wx.CallAfter(self.Refresh)
+
+    def setOpenSndCtrl(self, str):
+        self.openSndCtrl = str
+        self.OnResize(None)
+
     def SetValue(self, value):
         self.lastvalue = self.value
         value = clamp(value, self.minvalue, self.maxvalue)
@@ -399,11 +430,13 @@ class HSlider(Slider):
             dc.DrawBitmap(bitmario, self.pos-8, 0, True)
             self.mario += 1
             
-        if not self.midiLearn:    
-            dc.DrawLabel(self.midictl, wx.Rect(pos, 0, self.knobSize, h), wx.ALIGN_CENTER)
-        else:
+        if self.midiLearn:    
             dc.SetFont(wx.Font(LABEL_FONT-1, wx.NORMAL, wx.ITALIC, wx.LIGHT, face=FONT_FACE))
             dc.DrawLabel("Move a MIDI controller...", wx.Rect(5, 0, 50, h), wx.ALIGN_CENTER_VERTICAL)
+        elif self.openSndCtrl:
+            dc.DrawLabel(self.openSndCtrl, wx.Rect(5, 0, w, h), wx.ALIGN_CENTER_VERTICAL)            
+        else:
+            dc.DrawLabel(self.midictl, wx.Rect(5, 0, w, h), wx.ALIGN_CENTER_VERTICAL)
                 
         # Send value
         if self.outFunction:
@@ -411,16 +444,19 @@ class HSlider(Slider):
 
 class CECSlider:
     def __init__(self, parent, minvalue, maxvalue, init=None, label='slider', unit='', valtype='float', 
-                 log=False, name='', gliss=.025, midictl=None, tooltip='', up=False, function=None):
+                 log=False, name='', gliss=.025, midictl=None, tooltip='', up=False, half=False, function=None):
         self.widget_type = "slider"
         self.parent = parent
         self.valtype = valtype
         self.name = name
+        self.half = half
         self.function = function
         self.gliss = gliss
         self.automationLength = None
         self.automationData = []
         self.path = None
+        self.openSndCtrl = None
+        self.OSCOut = None
         self.midictl = None
         self.midichan = 1
         self.minvalue = minvalue
@@ -439,9 +475,13 @@ class CECSlider:
         if tooltip != '':
             self.slider.SetToolTip(wx.ToolTip(tooltip))
 
-        self.label = Label(parent, label, size=(120,16), outFunction=self.onLabelClick)
+        self.label = Label(parent, label, size=(100,16), outFunction=self.onLabelClick, dclickFunction=self.onLabelDClick)
         self.label.SetToolTip(CECTooltip(TT_SLIDER_LABEL))
-        self.entryUnit = EntryUnit(parent, self.slider.GetValue(), unit, size=(130,16),
+        if self.half:
+            unitX = 100
+        else:
+            unitX = 120
+        self.entryUnit = EntryUnit(parent, self.slider.GetValue(), unit, size=(unitX,16),
                                    valtype=valtype, outFunction=self.entryReturn)
         self.entryUnit.SetToolTip(CECTooltip(TT_SLIDER_DISPLAY))                           
         self.buttons = PlayRecButtons(parent, self, size=(40,16))
@@ -455,19 +495,34 @@ class CECSlider:
     def onLabelClick(self, label, shift=False, alt=False, side='left'):
         if not self.up:
             # alt is now the right click
-            if alt and shift:    
+            if alt and shift:
                 self.setMidiCtl(None)
             elif shift:
                 CeciliaLib.getVar("grapher").setShowLineSolo(label)
                 CeciliaLib.getVar("grapher").toolbar.menu.setLabel(label, True)
-            elif alt:    
-                CeciliaLib.getVar("grapher").toolbar.menu.setLabel(label, True)
-                CeciliaLib.getVar("audioServer").midiLearn(self)
-                self.slider.inMidiLearnMode()
+            elif alt:
+                if CeciliaLib.getVar("useMidi"):
+                    CeciliaLib.getVar("grapher").toolbar.menu.setLabel(label, True)
+                    CeciliaLib.getVar("audioServer").midiLearn(self)
+                    self.slider.inMidiLearnMode()
+                else:
+                    CeciliaLib.showErrorDialog("Midi not initialized!",
+                        "There is no Midi interface connected!")
             else:
                 CeciliaLib.getVar("grapher").resetShow()
                 CeciliaLib.getVar("grapher").toolbar.menu.setLabel(label, True)
 
+    def onLabelDClick(self, side='left'):
+        f = OSCPopupFrame(CeciliaLib.getVar('interface'), self)
+        f.CenterOnParent()
+        f.Show()
+
+    def setOSCInput(self, value):
+        self.setOpenSndCtrl(value)
+
+    def setOSCOutput(self, value):
+        self.setOSCOut(value)
+
     def setConvertSliderValue(self, x, end=None):
         self.convertSliderValue = x
 
@@ -536,14 +591,20 @@ class CECSlider:
         return self.buttons.getRec()
 
     def getState(self):
-        return [self.getValue(), self.getPlay(), self.getMidiCtl(), self.getMidiChannel()]
+        return [self.getValue(), self.getPlay(), self.getMidiCtl(), self.getMidiChannel(), self.getOpenSndCtrl(), self.getOSCOut()]
 
     def setState(self, values):
         self.setValue(values[0])
         self.setPlay(values[1])
-        self.setMidiCtl(values[2])
         if len(values) >= 4:
             self.setMidiChannel(values[3])
+        self.setMidiCtl(values[2])
+        if len(values) >= 5:
+            if values[4] != None:
+                self.setOpenSndCtrl("%d:%s" % (values[4][0], values[4][1]))
+        if len(values) >= 6:
+            if values[5] != None:
+                self.setOSCOut("%s:%d:%s" % (values[5][0], values[5][1], values[5][2]))
 
     def getPath(self):
         return self.path
@@ -555,7 +616,9 @@ class CECSlider:
             self.slider.setMidiCtl('')
         else:    
             self.midictl = int(ctl)
-            self.slider.setMidiCtl(str(self.midictl))
+            self.slider.setMidiCtl("%d:%d" % (self.midictl, self.midichan))
+            self.openSndCtrl = None
+            self.slider.setOpenSndCtrl('')
         self.slider.Refresh()
         
     def getMidiCtl(self):
@@ -572,7 +635,43 @@ class CECSlider:
             return True
         else:
             return False
-                
+
+    def setOpenSndCtrl(self, value):
+        if value != None:
+            if value == "":
+                self.openSndCtrl = None
+                self.slider.setOpenSndCtrl("")
+            else:
+                sep = value.split(":")
+                port = int(sep[0].strip())
+                address = str(sep[1].strip())
+                self.openSndCtrl = (port, address)
+                self.slider.setOpenSndCtrl("%d:%s" % (port, address))
+                self.setMidiCtl(None)
+
+    def setOSCOut(self, value):
+        if value != None:
+            if value == "":
+                self.OSCOut = None
+            else:
+                sep = value.split(":")
+                host = str(sep[0].strip())
+                port = int(sep[1].strip())
+                address = str(sep[2].strip())
+                self.OSCOut = (host, port, address)
+
+    def getOpenSndCtrl(self):
+        return self.openSndCtrl
+
+    def getOSCOut(self):
+        return self.OSCOut
+
+    def getWithOSC(self):
+        if self.openSndCtrl != None:
+            return True
+        else:
+            return False
+
     def setAutomationData(self, data):
         # convert values on scaling
         temp = []
@@ -696,7 +795,7 @@ class RangeSlider(wx.Panel):
             self.action = 'drag'
             self.handles = self.scale(self.handlePos)
             self.CaptureMouse()
-            self.Refresh()
+            wx.CallAfter(self.Refresh)
         
     def MouseDown(self, evt):
         size = self.GetSize()
@@ -715,7 +814,7 @@ class RangeSlider(wx.Panel):
             self.action = 'right'
         self.handles = self.scale(self.handlePos)
         self.CaptureMouse()
-        self.Refresh()
+        wx.CallAfter(self.Refresh)
 
     def MouseMotion(self, evt):
         size = self.GetSize()
@@ -731,7 +830,7 @@ class RangeSlider(wx.Panel):
             elif self.action == 'right':
                 self.handlePos[1] = clamp(xpos, self.handlePos[0]+1, size[0]-1)
             self.handles = self.scale(self.handlePos)
-            self.Refresh()
+            wx.CallAfter(self.Refresh)
 
     def MouseUp(self, evt):
         if self.HasCapture():
@@ -743,13 +842,13 @@ class RangeSlider(wx.Panel):
         self.createSliderBitmap()
         self.createBackgroundBitmap()
         self.clampHandlePos()
-        self.Refresh()
+        wx.CallAfter(self.Refresh)
 
     def clampHandlePos(self):
         size = self.GetSize()
         tmp = []
         for handle in [min(self.handles), max(self.handles)]:
-            pos = tFromValue(handle, self.minvalue, self.maxvalue) * (size[0])
+            pos = tFromValue(handle, self.minvalue, self.maxvalue) * size[0]
             pos = clamp(pos, 1, size[0]-1)
             tmp.append(pos)
         self.handlePos = tmp
@@ -764,14 +863,16 @@ class HRangeSlider(RangeSlider):
         self.clampHandlePos()
         self.midictl1 = ''
         self.midictl2 = ''
+        self.openSndCtrl1 = ''
+        self.openSndCtrl2 = ''
         self.midiLearn = False
-        self.font = wx.Font(LABEL_FONT, wx.NORMAL, wx.ITALIC, wx.LIGHT, face=FONT_FACE)
+        self.font = wx.Font(LABEL_FONT-2, wx.NORMAL, wx.ITALIC, wx.LIGHT, face=FONT_FACE)
 
     def setSliderHeight(self, height):
         self.sliderHeight = height
         self.createSliderBitmap()
         self.createBackgroundBitmap()
-        self.Refresh()
+        wx.CallAfter(self.Refresh)
 
     def createBackgroundBitmap(self):
         w,h = self.GetSize()
@@ -799,7 +900,28 @@ class HRangeSlider(RangeSlider):
 
     def inMidiLearnMode(self):
         self.midiLearn = True
-        self.Refresh()
+        wx.CallAfter(self.Refresh)
+
+    def setOpenSndCtrl(self, str, side):
+        if side == 'left':
+            self.openSndCtrl1 = str
+        else:
+            self.openSndCtrl2 = str
+        self.OnResize(None)
+
+    def SetOneValue(self, value, which):
+        self.lasthandles = self.handles
+        value = clamp(value, self.minvalue, self.maxvalue)
+        if self.log:
+            t = toLog(value, self.minvalue, self.maxvalue)
+            value = interpFloat(t, self.minvalue, self.maxvalue)
+        else:
+            t = tFromValue(value, self.minvalue, self.maxvalue)
+            value = interpFloat(t, self.minvalue, self.maxvalue)
+        if self.myType == IntType:
+            value = int(value)
+        self.handles[which] = value
+        self.OnResize(None)
 
     def SetValue(self, values):
         self.lasthandles = self.handles
@@ -852,12 +974,19 @@ class HRangeSlider(RangeSlider):
         rec = wx.Rect(mid-4, 4, 8, h-9)
         dc.DrawRoundedRectangleRect(rec, 3)
 
-        if not self.midiLearn:    
-            dc.DrawLabel(self.midictl1, wx.Rect(10, 0, 30, h), wx.ALIGN_CENTER_VERTICAL)
-            dc.DrawLabel(self.midictl2, wx.Rect(w-20, 0, 20, h), wx.ALIGN_CENTER_VERTICAL)
-        else:
+        if self.midiLearn:
             dc.SetFont(wx.Font(LABEL_FONT-1, wx.NORMAL, wx.ITALIC, wx.LIGHT, face=FONT_FACE))
             dc.DrawLabel("Move 2 MIDI controllers...", wx.Rect(5, 0, 50, h), wx.ALIGN_CENTER_VERTICAL)
+        elif self.openSndCtrl1 or self.openSndCtrl2:
+            if self.openSndCtrl1:
+                dc.DrawLabel(self.openSndCtrl1, wx.Rect(5, 0, w, h), wx.ALIGN_CENTER_VERTICAL)
+            if self.openSndCtrl2:
+                textwidth = dc.GetTextExtent(self.openSndCtrl2)[0] + 5
+                dc.DrawLabel(self.openSndCtrl2, wx.Rect(w-textwidth, 0, textwidth, h), wx.ALIGN_CENTER_VERTICAL)
+        else:
+            textwidth = dc.GetTextExtent(self.midictl2)[0] + 5
+            dc.DrawLabel(self.midictl1, wx.Rect(5, 0, 30, h), wx.ALIGN_CENTER_VERTICAL)
+            dc.DrawLabel(self.midictl2, wx.Rect(w-textwidth, 0, textwidth, h), wx.ALIGN_CENTER_VERTICAL)
 
         # Send value
         if self.outFunction:
@@ -882,6 +1011,8 @@ class CECRange:
         self.convertSliderValue = {'min': 200, 'max': 200}
         self.midictl = None
         self.midichan = [1,1]
+        self.openSndCtrl = None
+        self.OSCOut = None
 
         pos = (0,0)
         size = (200,16)
@@ -892,9 +1023,9 @@ class CECRange:
         if tooltip != '':
             self.slider.SetToolTip(wx.ToolTip(tooltip))
 
-        self.label = Label(parent, label, size=(120,16), outFunction=self.onLabelClick)
+        self.label = Label(parent, label, size=(100,16), outFunction=self.onLabelClick, dclickFunction=self.onLabelDClick)
         self.label.SetToolTip(CECTooltip(TT_RANGE_LABEL))
-        self.entryUnit = RangeEntryUnit(parent, self.slider.GetValue(), unit, size=(130,16), valtype=valtype, outFunction=self.entryReturn)
+        self.entryUnit = RangeEntryUnit(parent, self.slider.GetValue(), unit, size=(120,16), valtype=valtype, outFunction=self.entryReturn)
         self.entryUnit.SetToolTip(CECTooltip(TT_SLIDER_DISPLAY))                           
         self.buttons = PlayRecButtons(parent, self, size=(40,16))
         self.buttons.SetToolTip(CECTooltip(TT_SLIDER_PLAY + '\n\n' + TT_SLIDER_RECORD))
@@ -917,14 +1048,29 @@ class CECRange:
             elif shift:
                 CeciliaLib.getVar("grapher").setShowLineSolo(label)
                 CeciliaLib.getVar("grapher").toolbar.menu.setLabel(label, True)
-            elif rightclick:    
-                CeciliaLib.getVar("grapher").toolbar.menu.setLabel(label, True)
-                CeciliaLib.getVar("audioServer").midiLearn(self, True)
-                self.slider.inMidiLearnMode()
+            elif rightclick:
+                if CeciliaLib.getVar("useMidi"):
+                    CeciliaLib.getVar("grapher").toolbar.menu.setLabel(label, True)
+                    CeciliaLib.getVar("audioServer").midiLearn(self, True)
+                    self.slider.inMidiLearnMode()
+                else:
+                    CeciliaLib.showErrorDialog("Midi not initialized!",
+                        "There is no Midi interface connected!")
             else:
                 CeciliaLib.getVar("grapher").resetShow()
                 CeciliaLib.getVar("grapher").toolbar.menu.setLabel(label, True)
 
+    def onLabelDClick(self, side='left'):
+        f = OSCPopupFrame(CeciliaLib.getVar('interface'), self, side=side)
+        f.CenterOnParent()
+        f.Show()
+
+    def setOSCInput(self, value, side):
+        self.setOpenSndCtrl(value, side)
+
+    def setOSCOutput(self, value, side):
+        self.setOSCOut(value, side)
+
     def setConvertSliderValue(self, x, end='min'):
         self.convertSliderValue[end] = x
 
@@ -972,6 +1118,9 @@ class CECRange:
     def getUp(self):
         return self.up
 
+    def setOneValue(self, value, which):
+        self.slider.SetOneValue(value, which)
+
     def setValue(self, value):
         self.slider.SetValue(value)
 
@@ -1003,14 +1152,30 @@ class CECRange:
         return self.buttons.getRec()
 
     def getState(self):
-        return [self.getValue(), self.getPlay(), self.getMidiCtl(), self.getMidiChannel()]
+        return [self.getValue(), self.getPlay(), self.getMidiCtl(), self.getMidiChannel(), self.getOpenSndCtrl(), self.getOSCOut()]
 
     def setState(self, values):
         self.setValue(values[0])
         self.setPlay(values[1])
-        self.setMidiCtl(values[2])
         if len(values) >= 4:
             self.setMidiChannel(values[3])
+        self.setMidiCtl(values[2])
+        if len(values) >= 5:
+            if values[4] != None:
+                for i, tup in enumerate(values[4]):
+                    if tup != ():
+                        if i == 0:
+                            self.setOpenSndCtrl("%d:%s" % (tup[0], tup[1]), 'left')
+                        else:
+                            self.setOpenSndCtrl("%d:%s" % (tup[0], tup[1]), 'right')
+        if len(values) >= 6:
+            if values[5] != None:
+                for i, tup in enumerate(values[5]):
+                    if tup != ():
+                        if i == 0:
+                            self.setOSCOut("%s:%d:%s" % (tup[0], tup[1], tup[2]), 'left')
+                        else:
+                            self.setOSCOut("%s:%d:%s" % (tup[0], tup[1], tup[2]), 'right')
 
     def getPath(self):
         return self.path
@@ -1022,7 +1187,10 @@ class CECRange:
             self.slider.setMidiCtl('', '')
         else:    
             self.midictl = ctls
-            self.slider.setMidiCtl(str(self.midictl[0]), str(self.midictl[1]))
+            self.slider.setMidiCtl("%d:%d" % (self.midictl[0], self.midichan[0]), "%d:%d" % (self.midictl[1], self.midichan[1]))
+            self.openSndCtrl = None
+            self.slider.setOpenSndCtrl('', "left")
+            self.slider.setOpenSndCtrl('', "right")
         self.slider.Refresh()
 
     def getMidiCtl(self):
@@ -1040,6 +1208,72 @@ class CECRange:
         else:
             return False
 
+    def setOpenSndCtrl(self, value, side='left'):
+        if value != None:
+            if value == "":
+                self.slider.setOpenSndCtrl("", side)
+                if self.openSndCtrl != None:
+                    if side == 'left':
+                        self.openSndCtrl = ((), self.openSndCtrl[1])
+                    else:
+                        self.openSndCtrl = (self.openSndCtrl[0], ())
+                    if self.openSndCtrl == ((), ()):
+                        self.openSndCtrl = None
+            else:
+                self.setMidiCtl(None)
+                sep = value.split(":")
+                port = int(sep[0].strip())
+                address = str(sep[1].strip())
+                if self.openSndCtrl == None:
+                    if side == 'left':
+                        self.openSndCtrl = ((port, address), ())
+                    else:
+                        self.openSndCtrl = ((), (port, address))
+                else:
+                    if side == 'left':
+                        self.openSndCtrl = ((port, address), self.openSndCtrl[1])
+                    else:
+                        self.openSndCtrl = (self.openSndCtrl[0], (port, address))
+                self.slider.setOpenSndCtrl("%d:%s" % (port, address), side)
+
+    def setOSCOut(self, value, side='left'):
+        if value != None:
+            if value == "":
+                if self.OSCOut != None:
+                    if side == 'left':
+                        self.OSCOut = ((), self.OSCOut[1])
+                    else:
+                        self.OSCOut = (self.OSCOut[0], ())
+                    if self.OSCOut == ((), ()):
+                        self.OSCOut = None
+            else:
+                sep = value.split(":")
+                host = str(sep[0].strip())
+                port = int(sep[1].strip())
+                address = str(sep[2].strip())
+                if self.OSCOut == None:
+                    if side == 'left':
+                        self.OSCOut = ((host, port, address), ())
+                    else:
+                        self.OSCOut = ((), (host, port, address))
+                else:
+                    if side == 'left':
+                        self.OSCOut = ((host, port, address), self.OSCOut[1])
+                    else:
+                        self.OSCOut = (self.OSCOut[0], (host, port, address))
+                
+    def getOpenSndCtrl(self):
+        return self.openSndCtrl
+
+    def getOSCOut(self):
+        return self.OSCOut
+
+    def getWithOSC(self):
+        if self.openSndCtrl != None:
+            return True
+        else:
+            return False
+
     def setAutomationData(self, data, which=0):
         # convert values on scaling
         temp = []
@@ -1203,7 +1437,7 @@ class SplitterSlider(wx.Panel):
             return
         self.setHandlePosition(xpos)
         self.CaptureMouse()
-        self.Refresh()
+        wx.CallAfter(self.Refresh)
 
     def MouseMotion(self, evt):
         size = self.GetSize()
@@ -1211,7 +1445,7 @@ class SplitterSlider(wx.Panel):
             pos = evt.GetPosition()
             xpos = pos[0]
             self.setHandlePosition(xpos)
-            self.Refresh()
+            wx.CallAfter(self.Refresh)
 
     def MouseUp(self, evt):
         if self.HasCapture():
@@ -1223,7 +1457,7 @@ class SplitterSlider(wx.Panel):
         self.createSliderBitmap()
         self.createBackgroundBitmap()
         self.clampHandlePos()
-        self.Refresh()
+        wx.CallAfter(self.Refresh)
 
     def clampHandlePos(self):
         size = self.GetSize()
@@ -1253,7 +1487,7 @@ class HSplitterSlider(SplitterSlider):
         self.sliderHeight = height
         self.createSliderBitmap()
         self.createBackgroundBitmap()
-        self.Refresh()
+        wx.CallAfter(self.Refresh)
 
     def createBackgroundBitmap(self):
         w,h = self.GetSize()
@@ -1294,7 +1528,7 @@ class HSplitterSlider(SplitterSlider):
 
     def inMidiLearnMode(self):
         self.midiLearn = True
-        self.Refresh()
+        wx.CallAfter(self.Refresh)
 
     def SetValue(self, values):
         self.lasthandles = self.handles
@@ -1382,9 +1616,9 @@ class CECSplitter:
         if tooltip != '':
             self.slider.SetToolTip(wx.ToolTip(tooltip))
 
-        self.label = Label(parent, label, size=(120,16))
+        self.label = Label(parent, label, size=(100,16))
         self.label.SetToolTip(CECTooltip(TT_RANGE_LABEL))
-        self.entryUnit = SplitterEntryUnit(parent, self.slider.GetValue(), unit, size=(130,16), num=num_knobs, valtype=valtype, outFunction=self.entryReturn)
+        self.entryUnit = SplitterEntryUnit(parent, self.slider.GetValue(), unit, size=(120,16), num=num_knobs, valtype=valtype, outFunction=self.entryReturn)
         self.entryUnit.SetToolTip(CECTooltip(TT_SLIDER_DISPLAY))
         self.buttons = PlayRecButtons(parent, self, size=(40,16))
         self.buttons.SetToolTip(CECTooltip(TT_SLIDER_PLAY + '\n\n' + TT_SLIDER_RECORD))
@@ -1552,8 +1786,8 @@ class CECSplitter:
 
 def buildHorizontalSlidersBox(parent, list):
     mainBox = wx.BoxSizer(wx.VERTICAL)
-    box = wx.FlexGridSizer(24,4,2,5)
     sliders = []
+    halfcount = 0
     for widget in list:
         if widget['type'] in ['cslider', 'crange', 'csplitter']:
             name = widget['name']
@@ -1565,6 +1799,7 @@ def buildHorizontalSlidersBox(parent, list):
             init = widget['init']
             up = widget['up']
             midictl = widget['midictl']
+            half = widget['half']
             if midictl <= -1:
                 midictl = None
             valtype = widget['res']
@@ -1583,7 +1818,7 @@ def buildHorizontalSlidersBox(parent, list):
                 CeciliaLib.showErrorDialog('Error when building interface!', "'min' or 'max' arguments can't be 0 for a logarithmic slider. Reset to 'lin'.")
                 log = False
             if widget['type'] == 'cslider':
-                sl = CECSlider(parent, mini, maxi, init, label, unit, valtype, log, name, gliss, midictl, tooltip, up)
+                sl = CECSlider(parent, mini, maxi, init, label, unit, valtype, log, name, gliss, midictl, tooltip, up, half)
             elif widget['type'] == 'crange':
                 sl = CECRange(parent, mini, maxi, init, label, unit, valtype, log, name, gliss, midictl, tooltip, up)
             else:
@@ -1591,11 +1826,27 @@ def buildHorizontalSlidersBox(parent, list):
                 sl = CECSplitter(parent, mini, maxi, init, label, unit, valtype, num_knobs, log, name, gliss, midictl, tooltip, up)
             if up:
                 sl.buttons.Hide()
-            box.AddMany([(sl.label, 0, wx.LEFT, 5), (sl.buttons, 0, wx.LEFT, 0), 
-                         (sl.slider, 0, wx.EXPAND), (sl.entryUnit, 0, wx.LEFT | wx.RIGHT, 5)])
+            if not half:
+                box = wx.FlexGridSizer(1,4,2,5)
+                box.AddGrowableCol(2)
+                box.AddMany([(sl.label, 0, wx.LEFT, 5), (sl.buttons, 0, wx.LEFT, 0), 
+                             (sl.slider, 0, wx.EXPAND), (sl.entryUnit, 0, wx.LEFT | wx.RIGHT, 5)])
+                mainBox.Add(box, 1, wx.EXPAND | wx.TOP | wx.BOTTOM, 2)
+            else:
+                if halfcount % 2 == 0:
+                    lowerBox = wx.BoxSizer(wx.HORIZONTAL)
+                    leftbox, rightbox = wx.FlexGridSizer(1,4,2,5), wx.FlexGridSizer(1,4,2,5)
+                    leftbox.AddGrowableCol(2)
+                    rightbox.AddGrowableCol(2)
+                    leftbox.AddMany([(sl.label, 0, wx.LEFT, 5), (sl.buttons, 0, wx.LEFT, 0), 
+                                 (sl.slider, 0, wx.EXPAND), (sl.entryUnit, 0, wx.LEFT | wx.RIGHT, 5)])
+                    lowerBox.Add(leftbox, 1, wx.TOP | wx.BOTTOM, 0)
+                    mainBox.Add(lowerBox, 0, wx.EXPAND | wx.TOP | wx.BOTTOM, 2)
+                else:
+                    rightbox.AddMany([(sl.label, 0, wx.LEFT, 5), (sl.buttons, 0, wx.LEFT, 0), 
+                                 (sl.slider, 0, wx.EXPAND), (sl.entryUnit, 0, wx.LEFT | wx.RIGHT, 5)])
+                    lowerBox.Add(rightbox, 1, wx.TOP | wx.BOTTOM, 0)
+                halfcount += 1  
             sliders.append(sl)
 
-    box.AddGrowableCol(2)
-    mainBox.Add(box, 1, wx.EXPAND | wx.TOP | wx.BOTTOM, 5)
-
     return mainBox, sliders
