@@ -76,7 +76,7 @@ CeciliaVar['interfaceSize'] = (1000, 600)
 CeciliaVar['interfacePosition'] = None
 CeciliaVar['grapher'] = None
 CeciliaVar['gainSlider'] = None
-CeciliaVar['plugins'] = [None, None, None]
+CeciliaVar['plugins'] = [None] * NUM_OF_PLUGINS
 CeciliaVar['userSliders'] = []
 CeciliaVar['userTogglePopups'] = []
 CeciliaVar['userSamplers'] = []
@@ -96,13 +96,15 @@ CeciliaVar['startOffset'] = 0.0
 CeciliaVar['globalFade'] = 0.005
 CeciliaVar['audioServer'] = None
 CeciliaVar['automaticMidiBinding'] = 0
+CeciliaVar['showSpectrum'] = 0
+CeciliaVar['spectrumFrame'] = None
 
 # Server Flags
 CeciliaVar['sr'] = 44100
 CeciliaVar['nchnls'] = 2
 CeciliaVar['defaultNchnls'] = 2
 CeciliaVar['sampSize'] = 0
-CeciliaVar['audioFileType'] = 'aif' # aif, wav, 
+CeciliaVar['audioFileType'] = 'aif' # aif, wav, flac, ogg, sd2, au, caf
 CeciliaVar['samplePrecision'] = '32 bit' # '32 bit', '64 bit'
 CeciliaVar['bufferSize'] = '512'
 CeciliaVar['audioHostAPI'] = 'portaudio'
@@ -110,9 +112,12 @@ CeciliaVar['audioOutput'] = 0
 CeciliaVar['audioInput'] = 0
 CeciliaVar['enableAudioInput'] = 0
 CeciliaVar['useMidi'] = 0
+CeciliaVar['useSoundDur'] = 0
 CeciliaVar['midiPort'] = 'portmidi'
 CeciliaVar['midiDeviceIn'] = 0
-CeciliaVar['jack'] = {'client':'pyo'}
+CeciliaVar['defaultFirstInput'] = 0
+CeciliaVar['defaultFirstOutput'] = 0
+CeciliaVar['jack'] = {'client': 'cecilia5'}
 
 CeciliaVar['lastAudioFiles'] = ""
 
@@ -128,7 +133,8 @@ def readCeciliaPrefsFromFile():
         
         #### Some special cases ####
         convertToInt = ['sr', 'defaultNchnls', 'audioOutput', 'audioInput', 'sampSize', 'automaticMidiBinding',
-                        'midiDeviceIn', 'useTooltips', 'enableAudioInput', 'graphTexture']  
+                        'midiDeviceIn', 'useTooltips', 'enableAudioInput', 'graphTexture', 'showSpectrum', 'useSoundDur',
+                        'defaultFirstInput', 'defaultFirstOutput']  
         convertToFloat = ['defaultTotalTime', 'globalFade', 'DEBUG']                      
         convertToTuple = ['interfaceSize', 'interfacePosition']
         jackPrefs = ['client']
@@ -175,7 +181,8 @@ def writeCeciliaPrefsToFile():
                   'audioInput', 'midiPort', 'midiDeviceIn', 'samplePrecision', 'client', 'graphTexture', 
                   'globalFade', 'bufferSize', 'soundfilePlayer', 'soundfileEditor', 'prefferedPath', 'DEBUG',
                   'openFilePath', 'saveFilePath', 'saveAudioFilePath', 'openAudioFilePath', 'grapherLinePath',
-                  'defaultTotalTime', 'lastAudioFiles', 'automaticMidiBinding']
+                  'defaultTotalTime', 'lastAudioFiles', 'automaticMidiBinding', 'showSpectrum', 'useSoundDur',
+                  'defaultFirstInput', 'defaultFirstOutput']
     
     print('Writing Cecilia preferences...')
     
