@@ -25,8 +25,8 @@ reload(sys)
 sys.setdefaultencoding("utf-8")
 
 APP_NAME = 'Cecilia5'
-APP_VERSION = '5.0.8 beta'
-APP_COPYRIGHT = 'iACT,  2012'
+APP_VERSION = '5.0.9 beta'
+APP_COPYRIGHT = 'iACT,  2013'
 FILE_EXTENSION = "c5"
 PRESETS_DELIMITER = "####################################\n" \
                     "##### Cecilia reserved section #####\n" \
@@ -49,8 +49,8 @@ PREFERENCES_FILE = os.path.join(TMP_PATH, 'ceciliaPrefs.txt')
 LOG_FILE = os.path.join(TMP_PATH, 'ceciliaLog.txt')
 DOC_PATH = os.path.join(TMP_PATH, 'doc')
 MODULES_PATH = os.path.join(RESOURCES_PATH, 'modules')
-# Folder to save automations
 AUTOMATION_SAVE_PATH = os.path.join(TMP_PATH, 'automation_save')
+SPLASH_FILE_PATH = os.path.join(RESOURCES_PATH, "Cecilia_splash.png")
 
 # Meter icons
 ICON_VUMETER = catalog['vu-metre2.png']
@@ -58,6 +58,10 @@ ICON_VUMETER_DARK = catalog['vu-metre-dark2.png']
 # Plugin icons
 ICON_PLUGINS_KNOB = catalog['knob-trans-sm.png']
 ICON_PLUGINS_KNOB_DISABLE = catalog['knob-disab-sm.png']
+ICON_PLUGINS_ARROW_UP = catalog['arrow_up.png']
+ICON_PLUGINS_ARROW_UP_HOVER = catalog['arrow_up_hover.png']
+ICON_PLUGINS_ARROW_DOWN = catalog['arrow_down.png']
+ICON_PLUGINS_ARROW_DOWN_HOVER = catalog['arrow_down_hover.png']
 # Toolbox icons
 ICON_TB_LOAD = catalog['load-normal-trans.png']
 ICON_TB_LOAD_OVER = catalog['load-hover-trans.png']
@@ -122,6 +126,11 @@ ICON_PTB_RANDOM = catalog['random-normal-trans.png']
 ICON_PTB_RANDOM_OVER = catalog['random-hover-trans.png']
 ICON_PTB_WAVES = catalog['waves-normal-trans.png']
 ICON_PTB_WAVES_OVER = catalog['waves-hover-trans.png']
+# Input icons
+ICON_INPUT_1_FILE = catalog['input-1-file.png']
+ICON_INPUT_2_LIVE = catalog['input-2-live.png']
+ICON_INPUT_3_MIC = catalog['input-3-mic.png']
+ICON_INPUT_4_MIC_RECIRC = catalog['input-4-mic-recirc.png']
 # Crossfade icons
 ICON_XFADE_LINEAR = catalog['xfade-linear.png']
 ICON_XFADE_POWER = catalog['xfade-power.png']
@@ -153,13 +162,31 @@ MIDI_DRIVERS = ['portmidi']
 
 # plugin types
 PLUGINS_CHOICE = ['None', 'Reverb', 'WGVerb', 'Filter', 'Chorus', 'Para EQ', '3 Bands EQ', 'Compress', 'Gate', 
-                  'Disto', 'AmpMod', 'Phaser', 'Delay', 'Flange', 'Harmonizer', 'Resonators', 'DeadReson']
+                  'Disto', 'AmpMod', 'Phaser', 'Delay', 'Flange', 'Harmonizer', 'Resonators', 'DeadReson', 'ChaosMod']
+NUM_OF_PLUGINS = 4
 
 # Audio settings
-AUDIO_FILE_FORMATS = ['aif', 'wav']
 SAMPLE_RATES = ['22050','44100','48000', '88200', '96000']
 BIT_DEPTHS= {'16 bits int': 0, '24 bits int': 1, '32 bits int': 2, '32 bits float': 3}
 BUFFER_SIZES = ['64','128','256','512','1024','2048','4096','8192','16384']
+AUDIO_FILE_FORMATS = {'wav': 0, 'aif': 1, 'au': 2, 'sd2': 4, 'flac': 5, 'caf': 6, 'ogg': 7}
+AUDIO_FILE_WILDCARD =  "All files|*.*|" \
+            "Wave file|*.wave;*.WAV;*.WAVE;*.Wav;*.Wave;*.wav|" \
+            "AIFF file|*.aif;*.aiff;*.aifc;*.AIF;*.AIFF;*.Aif;*.Aiff|" \
+            "Flac file|*.flac;*.FLAC;*.Flac;|" \
+            "OGG file|*.ogg;*.OGG;*.Ogg;|" \
+            "SD2 file|*.sd2;*.SD2;*.Sd2;|" \
+            "AU file|*.au;*.AU;*.Au;|" \
+            "CAF file|*.caf;*.CAF;*.Caf"
+
+POLY_CHORDS = {'00 - None': [0], '06 - Major':[0,4,7,12], '07 - Minor':[0,3,7,12], '08 - Seventh':[0,4,7,10], 
+                '10 - Minor 7':[0,3,7,10], '09 - Major 7':[0,4,7,11],  '13 - Major 11':[0,4,7,11,18],
+                '15 - Minor 7b5':[0,3,6,10], '16 - Dimini.':[0,3,6,9], '12 - Minor 9':[0,3,7,10,14], 
+                '11 - Major 9':[0,4,7,11,14], '17 - Ninth':[0,4,7,10,14], '14 - Minor 11':[0,3,7,10,17], 
+                '04 - Serial':[0,1,2,3,4,5,6,7,8,9,10,11], '05 - Whole T.': [0,2,4,6,8,10], 
+                '01 - Phasing': [0,0.1291,0.1171,0.11832,0.125001,0.12799,0.11976,0.138711,0.12866,0.13681],
+                '02 - Chorus': [0,0.291,0.371,0.2832,0.35001,0.2799,0.2976,0.38711,0.3866,0.3681],
+                '03 - Detuned': [0,0.8291,0.9371,1.2832,1.35001,0.82799,0.92976,1.38711,1.3866,0.93681]}
 
 # Menu Ids
 ID_OPEN = 1002
@@ -175,12 +202,14 @@ ID_PLAY_STOP = 1034
 ID_BOUNCE = 1035
 ID_BATCH_FOLDER = 1036
 ID_BATCH_PRESET = 1037
+ID_USE_SOUND_DUR = 1040
 ID_USE_MIDI = 2052
 ID_MARIO = 3002
 ID_OPEN_RECENT = 4000
 ID_OPEN_BUILTIN = 4100
 ID_OPEN_AS_TEXT = 4500
 ID_UPDATE_INTERFACE = 4501
+ID_SHOW_SPECTRUM = 4550
 ID_MODULE_INFO = 4600
 ID_DOC_FRAME = 4601
 
@@ -232,6 +261,7 @@ GRADIENT_DARK_COLOUR = "#313740"
 WIDGET_BORDER_COLOUR = "#BBBBBB"
 KNOB_BORDER_COLOUR = "#929292"
 POPUP_BACK_COLOUR = "#80A0B0"
+POPUP_DISABLE_COLOUR = "#888888"
 POPUP_BORDER_COLOUR = "#222222"
 POPUP_LABEL_COLOUR = "#FFFFFF"
 POPUP_HIGHLIGHT_COLOR = "#DDDDDD"
@@ -271,6 +301,8 @@ TR_PLAY_NORMAL_COLOUR = '#009911'
 TR_PLAY_CLICK_COLOUR = '#007A29'
 TR_RECORD_OFF_COLOUR = '#6E3131'
 TR_RECORD_ON_COLOUR = '#FF0000'
+PREFS_FOREGROUND = '#222222'
+PREFS_PATH_BACKGROUND = '#AAAAAA'
 
 # Hue, Brightness, Saturation
 COLOUR_CLASSES = {'green': [100., 0.25, .75], 
@@ -292,23 +324,23 @@ COLOUR_CLASSES = {'green': [100., 0.25, .75],
         'filterred': [342., 0.44, .47], 
         'compblue': [240., 0.44, .22], 
         'grey': [0., 0.34, 0.],
-        'green1': [100., 0.25, .75], 
-        'green2': [85., 0.3, .6], 
+        'green1': [100., 0.25, .75], # filters popup and freq
+        'green2': [85., 0.3, .6], # filters Q
         'green3': [75., 0.4, .6],
         'green4': [65., 0.45, .4],
-        'blue1': [230., 0.35, .55], 
+        'blue1': [230., 0.35, .55], # dry/wet, amplitude, balance
         'blue2': [220., 0.4, .45], 
         'blue3': [203., 0.43, .4], 
         'blue4': [190., 0.5, .35],
-        'red1': [0., .28, .75],        
+        'red1': [0., .28, .75], # Pitch, transposition
         'red2': [355., 0.35, .65], 
         'red3': [350., 0.45, .55], 
         'red4': [345., 0.5, .45], 
-        'orange1': [22., 0.35, .85], 
+        'orange1': [22., 0.35, .85], # post-processing knob lines
         'orange2': [18., 0.4, .7], 
         'orange3': [16., 0.47, .55], 
         'orange4': [14., 0.52, .45],
-        'purple1': [290., 0.3, .65], 
+        'purple1': [290., 0.3, .65], # process specific parameters 
         'purple2': [280., 0.4, .6], 
         'purple3': [270., 0.47, .55], 
         'purple4': [260., 0.52, .45]
