@@ -26,11 +26,11 @@ def buildFileTree():
     root = MODULES_PATH
     directories = []
     files = {}
-    for dir in os.listdir(MODULES_PATH):
+    for dir in sorted(os.listdir(MODULES_PATH)):
         if not dir.startswith('.'):
             directories.append(dir)
             files[dir] = []
-            for f in os.listdir(os.path.join(root, dir)):
+            for f in sorted(os.listdir(os.path.join(root, dir))):
                 if not f.startswith('.'):
                     files[dir].append(f)
     return root, directories, files
@@ -81,9 +81,12 @@ class InterfaceMenuBar(wx.MenuBar):
                         except:
                             ok = False 
                         if ok:
-                            menu.Append(subId1, file)
-                            self.frame.Bind(wx.EVT_MENU, self.mainFrame.onOpenPrefModule, id=subId1)
-                            subId1 += 1
+                            try:
+                                menu.Append(subId1, CeciliaLib.ensureNFD(file))
+                                self.frame.Bind(wx.EVT_MENU, self.mainFrame.onOpenPrefModule, id=subId1)
+                                subId1 += 1
+                            except:
+                                pass
                 
         self.fileMenu.AppendMenu(-1, 'Modules', self.openBuiltinMenu)
 
@@ -102,7 +105,7 @@ class InterfaceMenuBar(wx.MenuBar):
         if recentFiles:
             for file in recentFiles:
                 try:
-                    self.openRecentMenu.Append(subId2, file)
+                    self.openRecentMenu.Append(subId2, CeciliaLib.ensureNFD(file))
                     subId2 += 1
                 except:
                     pass
@@ -145,24 +148,33 @@ class InterfaceMenuBar(wx.MenuBar):
         self.frame.Bind(wx.EVT_MENU, self.mainFrame.onRememberInputSound, id=ID_REMEMBER)
 
         # Action Options Menu
-        actionMenu = wx.Menu()
-        actionMenu.Append(ID_PLAY_STOP, 'Play / Stop\tCtrl+.', 'Start and stop audio server')
+        self.actionMenu = wx.Menu()
+        self.actionMenu.Append(ID_PLAY_STOP, 'Play / Stop\tCtrl+.', 'Start and stop audio server')
         self.frame.Bind(wx.EVT_MENU, self.mainFrame.onShortPlayStop, id=ID_PLAY_STOP)
-        actionMenu.AppendSeparator()
-        actionMenu.Append(ID_BOUNCE, 'Bounce to Disk\tCtrl+B', 'Record the audio processing in a soundfile')
+        self.actionMenu.AppendSeparator()
+        self.actionMenu.Append(ID_BOUNCE, 'Bounce to Disk\tCtrl+B', 'Record the audio processing in a soundfile')
         self.frame.Bind(wx.EVT_MENU, self.mainFrame.onBounceToDisk, id=ID_BOUNCE)
-        actionMenu.Append(ID_BATCH_FOLDER, 'Batch Processing on Sound Folder', '')
-        self.frame.Bind(wx.EVT_MENU, self.mainFrame.onBatchProcessing, id=ID_BATCH_FOLDER)
-        actionMenu.Append(ID_BATCH_PRESET, 'Batch Processing on Preset Sequence', '')
+        self.actionMenu.Append(ID_BATCH_PRESET, 'Batch Processing on Preset Sequence', '')
         self.frame.Bind(wx.EVT_MENU, self.mainFrame.onBatchProcessing, id=ID_BATCH_PRESET)
-        actionMenu.AppendSeparator()
-        actionMenu.Append(ID_USE_MIDI, 'Use MIDI', 'Allow Cecilia to use a midi device.', kind=wx.ITEM_CHECK)
+        self.actionMenu.Append(ID_BATCH_FOLDER, 'Batch Processing on Sound Folder', '')
+        self.frame.Bind(wx.EVT_MENU, self.mainFrame.onBatchProcessing, id=ID_BATCH_FOLDER)
+        self.actionMenu.Append(ID_USE_SOUND_DUR, 'Use Sound Duration on Folder Batch Processing', kind=wx.ITEM_CHECK)
+        if CeciliaLib.getVar("useSoundDur") == 1: 
+            self.actionMenu.FindItemById(ID_USE_SOUND_DUR).Check(True)
+        self.frame.Bind(wx.EVT_MENU, self.mainFrame.onUseSoundDuration, id=ID_USE_SOUND_DUR)
+        self.actionMenu.AppendSeparator()
+        self.actionMenu.Append(ID_SHOW_SPECTRUM, 'Show Spectrum', '', kind=wx.ITEM_CHECK)
+        self.frame.Bind(wx.EVT_MENU, self.mainFrame.onShowSpectrum, id=ID_SHOW_SPECTRUM)
+        if CeciliaLib.getVar('showSpectrum'):
+            self.actionMenu.FindItemById(ID_SHOW_SPECTRUM).Check(True)
+        self.actionMenu.AppendSeparator()
+        self.actionMenu.Append(ID_USE_MIDI, 'Use MIDI', 'Allow Cecilia to use a midi device.', kind=wx.ITEM_CHECK)
         if CeciliaLib.getVar("useMidi") == 1: midiCheck = True
         else: midiCheck = False
-        actionMenu.FindItemById(ID_USE_MIDI).Check(midiCheck)
-        actionMenu.FindItemById(ID_USE_MIDI).Enable(False)
+        self.actionMenu.FindItemById(ID_USE_MIDI).Check(midiCheck)
+        self.actionMenu.FindItemById(ID_USE_MIDI).Enable(False)
         self.frame.Bind(wx.EVT_MENU, self.mainFrame.onUseMidi, id=ID_USE_MIDI)
-    
+
         windowMenu = wx.Menu()
         windowMenu.Append(ID_MARIO, 'Eh Oh Mario!\tShift+Ctrl+E', '', kind=wx.ITEM_CHECK)
         self.frame.Bind(wx.EVT_MENU, self.marioSwitch, id=ID_MARIO)
@@ -177,10 +189,13 @@ class InterfaceMenuBar(wx.MenuBar):
  
         self.Append(self.fileMenu, '&File')
         self.Append(self.editMenu, '&Edit')
-        self.Append(actionMenu, '&Action')
+        self.Append(self.actionMenu, '&Action')
         self.Append(windowMenu, '&Window')
         self.Append(helpMenu, '&Help')
 
+    def spectrumSwitch(self, state):
+        self.actionMenu.FindItemById(ID_SHOW_SPECTRUM).Check(state)
+
     def marioSwitch(self, evt):
         if evt.GetInt() == 1:
             self.FindItemById(ID_MARIO).Check(1)
